.detail {
  margin: 24px;
}

.multiline {
  line-height: 140%;
}

.tint {
  /* Dark view_bg_color */
  background-color: rgb(30 30 30);
  color: white;
}

.search-row {
  padding: 2px;
}

.update {
  color: #ff7800;
}

comment-tile {
  padding: 16px;
}

.detail-heading {
  margin-bottom: 6px;
}

rating {
  color: var(--yellow-4);
}

info-bar.horizontal > :not(:first-child),
info-bar.horizontal > :not(:first-child) > row {
  border-radius: 0 12px 12px 0;
}

info-bar.horizontal > :not(:last-child),
info-bar.horizontal > :not(:last-child) > row {
  border-radius: 12px 0 0 12px;
}

info-bar.vertical > :not(:first-child),
info-bar.vertical > :not(:first-child) > row {
  border-radius: 0 0 12px 12px;
}

info-bar.vertical > :not(:last-child),
info-bar.vertical > :not(:last-child) > row {
  border-radius: 12px 12px 0 0;
}

info-bar > :not(:first-child):not(:last-child),
info-bar > :not(:first-child):not(:last-child) > row {
  border-radius: 0;
}

.card-separator {
  background: var(--card-shade-color);
}

.info-bar-item {
  background: transparent;
  box-shadow: 0 0 0 1px color-mix(in srgb, var(--card-shade-color), transparent);
  border-radius: 0;
  padding: 0;
}

.author-badge {
  background-color: var(--accent-color);
  border-radius: 999px;
  color: var(--window-bg-color);
  margin-left: 6px;
  padding: 2px 6px;
}

progressbar text {
  font-size: 14px;
  font-weight: bold;
}

/* Workaround for https://gitlab.gnome.org/GNOME/gtk/-/issues/5309 */
.boxed-list-placeholder > row:nth-last-child(2),
.boxed-list-placeholder > row.expander:nth-last-child(2):not(:checked) > box > list > row.header,
.boxed-list-placeholder > row.expander:nth-last-child(2) > box > revealer > list.nested > row:last-child {
  border-bottom: 0;
  border-bottom-left-radius: 12px;
  border-bottom-right-radius: 12px;
}

.boxed-list-placeholder > row.expander:nth-last-child(2) > box > revealer > list.nested {
  border-bottom-left-radius: 12px;
  border-bottom-right-radius: 12px;
}
